@@ -46,15 +46,15 @@ __all__ = [
     "get_properties_from_curves",
     "set_properties_on_curves",
     "set_y_axis_for_curve",
+    "CURVE_COLORS",
 ]
 
 import copy
 
 from taurus import warning
 from taurus.external.qt import Qt
-from taurus.core.util.containers import CaselessDict
 from taurus.qt.qtgui.util.ui import UILoadable
-from taurus_pyqtgraph.y2axis import Y2ViewBox, set_y_axis_for_curve
+from .y2axis import Y2ViewBox, set_y_axis_for_curve
 import pyqtgraph
 
 
@@ -82,19 +82,16 @@ for k, v in NamedLineStyles.items():
     ReverseNamedLineStyles[v] = k
 
 # TODO:allow the dialog to use this curve styles
-NamedCurveStyles = {
+NamedStepMode = {
     CONFLICT: "",
-    None: "",
-    "No curve": "No curve",
-    "Lines": "Lines",
-    "Sticks": "Sticks",
-    "Steps": "Steps",
-    "Dots": "Dots",
+    None: "No step",
+    "left": "Left step",
+    "right": "Right step",
 }
 
-ReverseNamedCurveStyles = {}
-for k, v in NamedCurveStyles.items():
-    ReverseNamedCurveStyles[v] = k
+ReverseNamedStepMode = {}
+for k, v in NamedStepMode.items():
+    ReverseNamedStepMode[v] = k
 
 NamedSymbolStyles = {
     CONFLICT: "",
@@ -117,17 +114,46 @@ for k, v in NamedSymbolStyles.items():
     ReverseNamedSymbolStyles[v] = k
 
 NamedColors = [
-    "Black",
     "Red",
     "Blue",
-    "Magenta",
     "Green",
+    "Magenta",
     "Cyan",
     "Yellow",
+    "Orange",
+    "greenyellow",
     "Gray",
     "White",
+    "Black",
 ]
 
+kelly_colors_hex = [  # https://stackoverflow.com/a/4382138
+    0xFFB300,  # Vivid Yellow
+    0x803E75,  # Strong Purple
+    0xFF6800,  # Vivid Orange
+    0xA6BDD7,  # Very Light Blue
+    0xC10020,  # Vivid Red
+    0xCEA262,  # Grayish Yellow
+    0x817066,  # Medium Gray
+    # The following don't work well for people with defective color vision
+    0x007D34,  # Vivid Green
+    0xF6768E,  # Strong Purplish Pink
+    0x00538A,  # Strong Blue
+    # 0xFF7A5C, # Strong Yellowish Pink
+    # 0x53377A, # Strong Violet
+    # 0xFF8E00, # Vivid Orange Yellow
+    0xB32851,  # Strong Purplish Red
+    # 0xF4C800, # Vivid Greenish Yellow
+    # 0x7F180D, # Strong Reddish Brown
+    0x93AA00,  # Vivid Yellowish Green
+    0x593315,  # Deep Yellowish Brown
+    # 0xF13A13, # Vivid Reddish Orange
+    # 0x232C16, # Dark Olive Green
+]
+
+CURVE_COLORS = [Qt.QColor(n) for n in NamedColors[:-2]]
+# CURVE_COLORS = [Qt.QColor(n) for n in kelly_colors_hex]
+
 
 @UILoadable
 class CurvesAppearanceChooser(Qt.QWidget):
@@ -167,18 +193,17 @@ class CurvesAppearanceChooser(Qt.QWidget):
 
         self.sStyleCB.insertItems(0, sorted(NamedSymbolStyles.values()))
         self.lStyleCB.insertItems(0, list(NamedLineStyles.values()))
-        self.cStyleCB.insertItems(0, list(NamedCurveStyles.values()))
+        self.stepModeCB.insertItems(0, list(NamedStepMode.values()))
         self.sColorCB.addItem("")
         self.lColorCB.addItem("")
         self.cAreaDSB.setRange(float("-inf"), float("inf"))
         if not showButtons:
             self.applyBT.hide()
             self.resetBT.hide()
-        for color in NamedColors:
+        for color in CURVE_COLORS:
             icon = self._colorIcon(color)
             self.sColorCB.addItem(icon, "", Qt.QColor(color))
             self.lColorCB.addItem(icon, "", Qt.QColor(color))
-        self.__itemsDict = CaselessDict()
         self.setCurvesProps(curvePropDict)
 
         if self.plotItem is None:
@@ -188,16 +213,16 @@ class CurvesAppearanceChooser(Qt.QWidget):
         self.curvesLW.itemSelectionChanged.connect(
             self._onSelectedCurveChanged
         )
-        self.curvesLW.itemChanged.connect(self._onItemChanged)
         self.applyBT.clicked.connect(self.onApply)
         self.resetBT.clicked.connect(self.onReset)
         self.sStyleCB.currentIndexChanged.connect(self._onSymbolStyleChanged)
 
+        self.curvesLW.itemChanged.connect(self._onControlChanged)
         self.sStyleCB.currentIndexChanged.connect(self._onControlChanged)
         self.lStyleCB.currentIndexChanged.connect(self._onControlChanged)
         self.sColorCB.currentIndexChanged.connect(self._onControlChanged)
         self.lColorCB.currentIndexChanged.connect(self._onControlChanged)
-        self.cStyleCB.currentIndexChanged.connect(self._onControlChanged)
+        self.stepModeCB.currentIndexChanged.connect(self._onControlChanged)
         self.sSizeSB.valueChanged.connect(self._onControlChanged)
         self.lWidthSB.valueChanged.connect(self._onControlChanged)
         self.cAreaDSB.valueChanged.connect(self._onControlChanged)
@@ -210,7 +235,7 @@ class CurvesAppearanceChooser(Qt.QWidget):
         self.bckgndBT.clicked.connect(self.changeBackgroundColor)
 
         # Disabled button until future implementations
-        self.changeTitlesBT.setEnabled(False)
+        self.changeTitlesBT.setVisible(False)
 
         # disable the group box with the options for swap curves between Y axes
         if Y2Axis is None or plotItem is None:
@@ -227,6 +252,14 @@ class CurvesAppearanceChooser(Qt.QWidget):
         if checked:
             self.assignToY1BT.setChecked(False)
 
+    def __findCurveListItem(self, key, role=None):
+        if role is None:
+            role = self.NAME_ROLE
+        for i in range(self.curvesLW.count()):
+            item = self.curvesLW.item(i)
+            if item.data(role) == key:
+                return item
+
     def changeBackgroundColor(self):
         """Launches a dialog for choosing the plot widget background color
         """
@@ -249,13 +282,10 @@ class CurvesAppearanceChooser(Qt.QWidget):
         self.curvePropDict = curvePropDict
         self._curvePropDictOrig = copy.deepcopy(curvePropDict)
         self.curvesLW.clear()
-        self.__itemsDict = CaselessDict()
         for name, prop in self.curvePropDict.items():
             # create and insert the item
             item = Qt.QListWidgetItem(prop.title, self.curvesLW)
-            self.__itemsDict[name] = item
             item.setData(self.NAME_ROLE, name)
-            item.setToolTip("<b>Curve Name:</b> %s" % name)
             item.setFlags(
                 Qt.Qt.ItemIsEnabled
                 | Qt.Qt.ItemIsSelectable
@@ -265,28 +295,6 @@ class CurvesAppearanceChooser(Qt.QWidget):
             )
         self.curvesLW.setCurrentRow(0)
 
-    def _onItemChanged(self, item):
-        """slot used when an item data has changed"""
-        name = item.data(self.NAME_ROLE)
-        previousTitle = self.curvePropDict[name].title
-        currentTitle = item.text()
-        if previousTitle != currentTitle:
-            self.curvePropDict[name].title = currentTitle
-            self.CurveTitleEdited.emit(name, currentTitle)
-
-    def updateTitles(self, newTitlesDict=None):
-        """
-        Updates the titles of the curves that are displayed in the curves list.
-
-        :param newTitlesDict: (dict<str,str>) dictionary with key=curve_name
-                                and value=title
-        """
-        if newTitlesDict is None:
-            return
-        for name, title in newTitlesDict.items():
-            self.curvePropDict[name].title = title
-            self.__itemsDict[name].setText(title)
-
     def getSelectedCurveNames(self):
         """Returns the curve names for the curves selected at the curves list.
 
@@ -318,8 +326,8 @@ class CurvesAppearanceChooser(Qt.QWidget):
         self.lStyleCB.setCurrentIndex(
             self.lStyleCB.findText(NamedLineStyles[prop.lStyle])
         )
-        self.cStyleCB.setCurrentIndex(
-            self.cStyleCB.findText(NamedCurveStyles[prop.cStyle])
+        self.stepModeCB.setCurrentIndex(
+            self.stepModeCB.findText(NamedStepMode[prop.stepMode])
         )
 
         if prop.y2 is CONFLICT:
@@ -426,28 +434,27 @@ class CurvesAppearanceChooser(Qt.QWidget):
         """
         text = str(text)
         if self.sSizeSB.value() < 2 and text not in ["", "No symbol"]:
-            # a symbol size of 0 is invisible and 1 means you should use
-            # cStyle=dots
             self.sSizeSB.setValue(3)
 
     def getShownProperties(self):
         """Returns a copy of the currently shown properties and updates
         self._shownProp
 
+        Note: the title property is left as CONFLICT since all are shown
+
         :return: (CurveAppearanceProperties)
         """
         prop = CurveAppearanceProperties()
 
-        for name in self.getSelectedCurveNames():
-            prop.title = self.curvePropDict[name].title
-
         # get the values from the Style comboboxes. Note that the empty string
         # ("") translates into CONFLICT
         prop.sStyle = ReverseNamedSymbolStyles[
             str(self.sStyleCB.currentText())
         ]
         prop.lStyle = ReverseNamedLineStyles[str(self.lStyleCB.currentText())]
-        prop.cStyle = ReverseNamedCurveStyles[str(self.cStyleCB.currentText())]
+        prop.stepMode = ReverseNamedStepMode[
+            str(self.stepModeCB.currentText())
+        ]
         # get sSize and lWidth from the spinboxes (-1 means conflict)
         v = self.sSizeSB.value()
         if v == -1:
@@ -525,6 +532,8 @@ class CurvesAppearanceChooser(Qt.QWidget):
                 [self.curvePropDict[n], prop],
                 conflict=CurveAppearanceProperties.inConflict_update_a,
             )
+            # update the title with whatever is now written in the curvesLW
+            self.curvePropDict[n].title = self.__findCurveListItem(n).text()
         # emit a (PyQt) signal telling what properties (first argument) need to
         # be applied to which curves (second argument)
         # self.curveAppearanceChanged.emit(prop, names)
@@ -587,7 +596,7 @@ def get_properties_from_curves(curves):
         lStyle = pen.style()
         lWidth = pen.width()
         lColor = pen.color()
-        cStyle = None
+        stepMode = opts.get("stepMode", None)
 
         cFill = opts["fillLevel"]
 
@@ -599,7 +608,7 @@ def get_properties_from_curves(curves):
             lStyle=lStyle,
             lWidth=lWidth,
             lColor=lColor,
-            cStyle=cStyle,
+            stepMode=stepMode,
             cFill=cFill,
             y2=y2,
             title=title,
@@ -620,7 +629,7 @@ def set_properties_on_curves(properties, curves, plotItem=None, y2Axis=None):
                        given curve
     :param curves: dict whose values are :class:`PlotDataItem` instances
                    and whose keys match those of properties (if a key in
-                   `curves` does not exist in `properties`, it will bed)
+                   `curves` does not exist in `properties`, it will be ignored)
     :param plotItem: The :class:`PlotItem` containing the dataItem.
     :param y2Axis: The :class:`Y2ViewBox` instance
  e skipp   """
@@ -639,8 +648,9 @@ def set_properties_on_curves(properties, curves, plotItem=None, y2Axis=None):
         lWidth = prop.lWidth
         lColor = prop.lColor
         cFill = prop.cFill
+        stepMode = prop.stepMode
         y2 = prop.y2
-        # title = properties.title
+        title = prop.title
 
         dataItem.setPen(dict(style=lStyle, width=lWidth, color=lColor))
         if cFill is not None:
@@ -666,6 +676,23 @@ def set_properties_on_curves(properties, curves, plotItem=None, y2Axis=None):
         else:
             dataItem.setSymbolBrush(None)
 
+        dataItem.opts["stepMode"] = stepMode
+        dataItem.updateItems()
+
+        if title is not None:
+            # set the title of the curve
+            dataItem.setData(name=title)
+            # update the corresponding label in the legend
+            if plotItem is not None and plotItem.legend is not None:
+                if hasattr(plotItem.legend, "getLabel"):
+                    plotItem.legend.getLabel(dataItem).setText(title)
+                else:
+                    # workaround for pyqtgraph<=0.11 (getLabel not implemented)
+                    for sample, label in plotItem.legend.items:
+                        if sample.item == dataItem:
+                            label.setText(title)
+                            break
+
         # act on the ViewBoxes only if plotItem and y2Axis are given
         if plotItem and y2Axis:
             set_y_axis_for_curve(y2, dataItem, plotItem, y2Axis)
@@ -682,7 +709,7 @@ class CurveAppearanceProperties(object):
         "lStyle",
         "lWidth",
         "lColor",
-        "cStyle",
+        "stepMode",
         "cFill",
         "y2",
         "title",
@@ -698,7 +725,7 @@ class CurveAppearanceProperties(object):
         lStyle=CONFLICT,
         lWidth=CONFLICT,
         lColor=CONFLICT,
-        cStyle=CONFLICT,
+        stepMode=CONFLICT,
         y2=CONFLICT,
         cFill=CONFLICT,
         title=CONFLICT,
@@ -713,7 +740,7 @@ class CurveAppearanceProperties(object):
             - lStyle= linestyle
             - lWidth= int
             - lColor= color
-            - cStyle= curvestyle
+            - stepMode= stepmode
             - cFill= float or None
             - y2= bool
             - visible = bool
@@ -724,7 +751,7 @@ class CurveAppearanceProperties(object):
               Qt.Qt.GlobalColor, a color name, or a Qt.Qcolor)
             - symbolstyle is a key of NamedSymbolStyles
             - linestyle is a key of Qt.Qt.PenStyle
-            - curvestyle is a key of NamedCurveStyles
+            - stepMode is a key of NamedStepMode
             - cFill can either be None (meaning not to fill) or a float that
               indicates the baseline from which to fill
             - y2 is True if the curve is associated to the y2 axis
@@ -736,7 +763,7 @@ class CurveAppearanceProperties(object):
         self.lStyle = lStyle
         self.lWidth = lWidth
         self.lColor = lColor
-        self.cStyle = cStyle
+        self.stepMode = stepMode
         self.cFill = cFill
         self.y2 = y2
         self.title = title
