@@ -35,6 +35,7 @@ from taurus.qt.qtgui.panel import TaurusModelChooser
 from taurus_pyqtgraph.taurusimageitem import TaurusImageItem
 from taurus_pyqtgraph.taurusplotdataitem import TaurusPlotDataItem
 from taurus_pyqtgraph.curvesmodel import TaurusItemConf, TaurusItemConfDlg
+from taurus_pyqtgraph.util import ensure_unique_curve_name
 import taurus
 from collections import OrderedDict
 from taurus.qt.qtcore.mimetypes import (
@@ -297,16 +298,24 @@ class TaurusXYModelChooserTool(Qt.QAction, BaseConfigurableClass):
 
     def _onTriggered(self):
         oldconfs = self._getTaurusPlotDataItemConfigs().values()
-        newconfs, ok = TaurusItemConfDlg.showDlg(
+        dlg = Qt.QDialog(self.parent())
+        dlg.setWindowTitle("Curves Selection")
+        layout = Qt.QVBoxLayout()
+        w = TaurusItemConfDlg(
             parent=self.parent(),
-            taurusItemConf=oldconfs,
-            showXCol=self._showX,
+            taurusItemsConf=oldconfs,
+            showXcol=self._showX,
         )
+        layout.addWidget(w)
+        dlg.setLayout(layout)
+        w.applied.connect(self._onDlgApplied)
+        w.ui.cancelBT.clicked.connect(dlg.close)
+        dlg.exec_()
 
-        if ok:
-            xy_names = [(c.xModel, c.yModel) for c in newconfs]
-            self.updateModels(xy_names)
-            # TODO: apply configurations too
+    def _onDlgApplied(self):
+        newconfs = self.sender().getItemConfs()
+        xy_names = [(c.xModel, c.yModel) for c in newconfs]
+        self.updateModels(xy_names)
 
     def _getTaurusPlotDataItemConfigs(self):
         """Get all the TaurusItemConf of the existing TaurusPlotDataItems
@@ -457,9 +466,13 @@ class TaurusXYModelChooserTool(Qt.QAction, BaseConfigurableClass):
             else:
                 if cname is None:
                     cname = ymodel.getSimpleName()
-                item = self.itemClass(xModel=xname, yModel=yname, name=cname)
-                if self._curveColors is not None:
-                    item.setPen(self._curveColors.next().color())
+                item = self.itemClass(
+                    xModel=xname,
+                    yModel=yname,
+                    name=cname,
+                    colors=self._curveColors,
+                )
+                item = ensure_unique_curve_name(item, self.plot_item)
                 self.plot_item.addItem(item)
 
 
